@@ -20,7 +20,7 @@ import gtk
 import gobject
 
 from gazpacho.loader import tags
-from gazpacho.properties import prop_registry, UNICHAR_PROPERTIES
+from gazpacho.properties import PropertyCustomEditor, UNICHAR_PROPERTIES, prop_registry 
 from gazpacho.signaleditor import SignalEditor
 
 class IncompleteFunctions(Exception):
@@ -547,8 +547,6 @@ class EditorPropertySpecial(EditorProperty):
         self.input = self.custom_editor.get_editor_widget()
         
     def load(self, widget):
-        context = self._app.get_current_context()
-        
         self._start_load(widget)
         
         self.widget = widget
@@ -638,6 +636,85 @@ class EditorPropertyObject(EditorProperty):
         widget = model[active_iter][1]
         self.set(widget)
 
+class PropertyEditorDialog(gtk.Dialog):
+    """This is a base abstract class for those editors that need to be
+    opened in a dialog.
+    """
+
+    def __init__(self):
+        """The widgets of the dialog are created in the virtual method
+        _create_widgets.
+        """
+        gtk.Dialog.__init__(self,
+                            title='',
+                            parent=None,
+                            flags=gtk.DIALOG_DESTROY_WITH_PARENT,
+                            buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
+        self.widget = None
+        self.gwidget = None
+        self.proxy = None
+        
+        # avoid destrcution when clicking the x button
+        self.connect('delete-event', self._on_delete_event)
+        self.connect('response', self._on_response)
+
+        self.set_border_width(6)
+        self.set_resizable(False)
+        self.set_has_separator(False)
+        self.vbox.set_spacing(6)
+        
+        self._create_widgets()
+
+        self.vbox.show_all()
+
+    def _create_widgets(self):
+        """Subclasses should override this."""
+
+    def set_widget(self, widget, proxy):
+        """Every time the user clicks on a custom property editor button
+        we are called with a different widget and proxy.
+        """
+        from gazpacho.widget import Widget
+
+        self.widget = widget
+        self.proxy = proxy
+        self.gwidget = Widget.from_widget(widget)
+
+    def _on_delete_event(self, dialog, event):
+        self.hide()
+        return True
+    
+    def _on_response(self, dialog, response_id):
+        self.hide()
+
+class PropertyCustomEditorWithDialog(PropertyCustomEditor):
+    """A custom editor which is just a button that opens up a dialog"""
+
+    # subclasses should probably want to override the dialog_class
+    dialog_class = PropertyEditorDialog
+    button_text = _('Edit...')
+    
+    def __init__(self):
+        self.dialog = self.dialog_class()
+        self.input = gtk.Button(self.button_text)
+        self.input.connect('clicked', self.on_button_clicked)
+        
+        self.application_window = None
+        self.proxy = None
+        self.edited_widget = None
+
+    def get_editor_widget(self):
+        return self.input
+
+    def update(self, context, gtk_widget, proxy):
+        self.application_window = context.get_application_window()
+        self.edited_widget = gtk_widget
+        self.proxy = proxy
+        
+    def on_button_clicked(self, button):
+        self.dialog.set_transient_for(self.application_window)
+        self.dialog.set_widget(self.edited_widget, self.proxy)
+        self.dialog.present()
 
 # this maps property types with EditorProperty classes
 property_map = {
@@ -679,7 +756,7 @@ class EditorPage(gtk.Table):
         self._rows += 1
 
     def _create_editor_property(self, property_class, app):
-        if property_class.editor:
+        if property_class.editor != PropertyCustomEditor:
             return EditorPropertySpecial(property_class, app)
 
         # Special case GObject subclasses
@@ -697,7 +774,7 @@ class EditorPage(gtk.Table):
             #                 property_class.tooltip)
 
             description = editor.description
-            if property_class.editor and property_class.editor.wide_editor:
+            if property_class.editor.wide_editor:
                 # no label for this editor since it need both columns
                 # of the table
                 description = None
@@ -710,7 +787,7 @@ class EditorPage(gtk.Table):
         # Class
         label = gtk.Label(_('Class')+':')
         label.set_alignment(1.0, 0.5)
-        class_label = gtk.Label(widget_adaptor.type_name)
+        class_label = gtk.Label(widget_adaptor.editor_name)
         class_label.set_alignment(0.0, 0.5)
         self._attach_row(label, class_label)
 
@@ -738,13 +815,10 @@ class Editor(gtk.Notebook):
         # if no widgets are selected
         self._loaded_widget = None
 
-        # A dictionary of properties for which we have a widget, eg they are
-        # displayed with an editor
-        self._widget_properties = {}
+        # A list of properties for the current widget
+        # XXX: We might want to cache this
+        self._widget_properties = []
         
-        # A reference to the loaded widget adaptor
-        self._loaded_adaptor = None
-
         self._signal_editor = None
 
         self._tooltips = gtk.Tooltips()
@@ -784,8 +858,9 @@ class Editor(gtk.Notebook):
         
         # Put the property widgets on the right page
         widget_properties = []
+        parent = widget.gtk_widget.get_parent()
         for property_class in prop_registry.list(adaptor.type_name,
-                                                 widget.gtk_widget.get_parent()):
+                                                 parent):
 
             if not property_class.editable:
                 continue
@@ -804,7 +879,7 @@ class Editor(gtk.Notebook):
             if property_widget:
                 widget_properties.append(property_widget)
                 
-        self._widget_properties[adaptor] = widget_properties
+        self._widget_properties = widget_properties
 
         # XXX: Remove this, show all widgets individually instead
         widget_table.show_all()
@@ -818,14 +893,42 @@ class Editor(gtk.Notebook):
         self._signal_editor = SignalEditor(self, self._app)
         self._vbox_signals.pack_start(self._signal_editor)
 
+    def _get_parent_types(self, widget):
+        retval = [type(widget)]
+        while True:
+            parent = widget.get_parent()
+            if not parent:
+                return retval
+            retval.append(type(parent))
+            widget = parent
+    
+    def _needs_rebuild(self, widget):
+        """
+        Return True if we need to rebuild the current property pages, False
+        if it's not require.
+        """
+        
+        if not self._loaded_widget:
+            return True
+
+        # Check if we need to rebuild the interface, otherwise we might end
+        # up with a (child) properties which does not belong to us
+        # FIXME: This implementation is not optimal, in some cases it'll
+        # rebuild even when it doesn't need to, a better way would be
+        # to compare child properties.
+        if (self._get_parent_types(self._loaded_widget.gtk_widget) !=
+            self._get_parent_types(widget.gtk_widget)):
+            return True
+
+        return False
+        
     def _load_widget(self, widget):
-        if self._loaded_adaptor != widget.adaptor:
+        if self._needs_rebuild(widget):
             self.clear()
             self._create_property_pages(widget, widget.adaptor)
             self._create_signal_page()
-            self._loaded_adaptor = widget.adaptor
 
-        for widget_property in self._widget_properties[widget.adaptor]:
+        for widget_property in self._widget_properties:
             widget_property.load(widget)
         self._signal_editor.load_widget(widget)
 
@@ -838,7 +941,6 @@ class Editor(gtk.Notebook):
                      self._vbox_packing):
             map(vbox.remove, vbox.get_children())
         self._loaded_widget = None
-        self._loaded_adaptor = None
  
     def display(self, widget):
         "Display a widget in the editor"
