@@ -43,46 +43,74 @@ def run_batch(app, command):
     ns = dict(new=lambda w: app.create(w))
     eval(command, {}, ns)
 
-def show_debug():
-    print 'Loading PyGTK...'
+def run_console(app):
+    try:
+        import readline
+        import rlcompleter
+        assert rlcompleter
+        readline.parse_and_bind('tab: complete')
+    except ImportError:
+        pass
+    
+    project = app.get_current_project()
+
+    import code
+    ia = code.InteractiveConsole(locals=locals())
+    ia.interact()
+    
+def check_gtk(debug):
+    if debug:
+        print 'Loading PyGTK...'
+    
+    try:
+        import gtk
+    except ImportError:
+        try:
+            import pygtk
+            # This modifies sys.path
+            pygtk.require('2.0') 
+            # Try again now when pygtk is imported
+            import gtk
+        except ImportError:
+            raise SystemExit("PyGTK is required to run Gazpacho")
+        
     import gobject
-    import gtk
     if (os.path.dirname(os.path.dirname(gtk.__file__)) !=
         os.path.dirname(gobject.__file__)):
-        print ('WARNING: Gtk and GObject modules are not loaded from '
+        print ('WARNING: GTK+ and GObject modules are not loaded from '
                'the same prefix')
-        
-    print 'Python:\t', '.'.join(map(str, sys.version_info[:3]))
-    print 'GTK:\t', '.'.join(map(str, gtk.gtk_version))
-    print 'PyGTK:\t', '.'.join(map(str, gtk.pygtk_version))
-    
-def launch(filenames=[], debug=False, profile=False,
-           batch=None, update=None):
-    if debug:
-        show_debug()
 
-    import gtk
+    if debug:
+        print 'Python:\t', '.'.join(map(str, sys.version_info[:3]))
+        print 'GTK+:\t', '.'.join(map(str, gtk.gtk_version))
+        print 'PyGTK:\t', '.'.join(map(str, gtk.pygtk_version))
+
     if gtk.pygtk_version < PYGTK_REQUIRED:
         raise SystemExit("PyGTK 2.6.0 or higher required to run Gazpacho")
 
-    # Ignore deprecation warnings when using 2.7.0
     if gtk.pygtk_version >= (2, 7, 0):
         if gtk.pygtk_version in ((2, 7, 0), (2, 7, 1)):
-            raise SystemExit("PyGtk 2.7.0 and 2.7.1 are buggy "
+            raise SystemExit("PyGTK 2.7.0 and 2.7.1 are buggy "
                              "Upgrade to 2.7.2 or downgrade to 2.6.x")
         
         if debug:
             print 'Using PyGTK 2.7.x, ignoring deprecation warnings'
+            
+        # Ignore deprecation warnings when using 2.7.x
         import warnings
         warnings.filterwarnings('ignore', category=DeprecationWarning)
-
+    
+def launch(options, filenames=[]):
+    # Do this before importing application, which imports gtk
+    check_gtk(debug=options.debug)
+    
     # Delay imports, so command line parsing is not slowed down
     from gazpacho.application import Application
     from gazpacho.debugwindow import DebugWindow, show
     
     gazpacho = Application()
 
-    if not debug:
+    if not options.debug:
         DebugWindow.application = gazpacho
         sys.excepthook = show
 
@@ -93,9 +121,9 @@ def launch(filenames=[], debug=False, profile=False,
         if not os.access(filename, os.R_OK):
             raise SystemExit('Could not open file %s: Permission denied.' %
                              filename)
-        open_project(gazpacho, filename, profile)
+        open_project(gazpacho, filename, options.profile)
 
-    if update:
+    if options.update:
         for project in gazpacho.get_projects():
             project.save(project.path)
 
@@ -105,18 +133,20 @@ def launch(filenames=[], debug=False, profile=False,
     if not filenames:
         gazpacho.new_project()
     
-    if batch:
-        run_batch(gazpacho, batch)
+    if options.batch:
+        run_batch(gazpacho, options.batch)
+        return
+    elif options.console:
+        run_console(gazpacho)
         return
 
-    gazpacho.show_all()
-
-    gtk.main()
+    gazpacho.run()
 
 def main(args=[]):
     from gazpacho.environ import environ
     gettext.bindtextdomain('gazpacho', environ.get_languages_dir())
-    gettext.bind_textdomain_codeset('utf-8')
+    # When is this required, seems to work fine without in Python 2.[34]
+    #gettext.bind_textdomain_codeset('utf-8')
     gettext.textdomain('gazpacho')
  
     parser = optparse.OptionParser()
@@ -136,15 +166,18 @@ def main(args=[]):
                       action="store_true", 
                       dest="update",
                       help=_("Load glade file and save it"))
+    parser.add_option('-c', '--console',
+                      action="store_true", 
+                      dest="console",
+                      help=_("Start up a console"))
     
     options, args = parser.parse_args(args)
 
-    if options.batch:
+    if options.batch or options.console:
         options.debug = True
         
     filenames = []
     if len(args) >= 2:
         filenames = [os.path.abspath(name) for name in args[1:]]
 
-    launch(filenames, options.debug, options.profile,
-           options.batch, options.update)
+    launch(options, filenames)
