@@ -68,10 +68,6 @@ class SimpleLoaderTest(BaseTest):
     def testEmpty(self):
         self.assertRaises(TypeError, ObjectBuilder, buffer="")
         
-    def testSpaces(self):
-        ob = self.openraw(" ") 
-        self.assertEqual(len(ob), 0)
-
     def testSimple(self):
         ob = self.openraw("<glade-interface></glade-interface>")
         self.assertEqual(len(ob), 0)
@@ -592,7 +588,7 @@ class CustomTest(BaseTest):
         ob = self.open(self.data, custom=self)
         self.assertEqual(self.ran, True)
 
-    def func(self, name, string1, string2, int1, int2):
+    def func(self, name, string1, string2, int1, int2, **kwargs):
         self.assertEqual(name, 'custom1')
         self.assertEqual(string1, 'foo')
         self.assertEqual(string2, 'bar')
